@@ -52,6 +52,7 @@
 #include "pdns.h"
 #include "sort.h"
 #include "time.h"
+#include "tokstr.h"
 #include "globals.h"
 #undef MAIN_PROGRAM
 
@@ -356,11 +357,10 @@ main(int argc, char *argv[]) {
 			if (sorting == no_sort)
 				usage("-k must be preceded by -s or -S");
 
-			char *saveptr = NULL;
-			const char *tok;
-			for (tok = strtok_r(optarg, ",", &saveptr);
-			     tok != NULL;
-			     tok = strtok_r(NULL, ",", &saveptr))
+			struct tokstr *ts = tokstr_string(optarg);
+			for (char *tok;
+			     (tok = tokstr_next(ts, ",")) != NULL;
+			     free(tok))
 			{
 				if (find_sort_key(tok) != NULL)
 					usage("Each sort key may only be "
@@ -369,6 +369,7 @@ main(int argc, char *argv[]) {
 				if ((msg = add_sort_key(tok)) != NULL)
 					usage(msg);
 			}
+			tokstr_last(&ts);
 			break;
 		    }
 		case 'J':
@@ -402,9 +403,11 @@ main(int argc, char *argv[]) {
 			}
 			break;
 		case 'T': {
-			char *copy, *walker, *token;
-			copy = walker = strdup(optarg);
-			while ((token = strsep(&walker, ",")) != NULL)
+			struct tokstr *ts = tokstr_string(optarg);
+			for (char *token;
+			     (token = tokstr_next(ts, ",")) != NULL;
+			     free(token))
+			{
 				if (strcasecmp(token, "reverse") == 0)
 					transforms |= TRANS_REVERSE;
 				else if (strcasecmp(token, "datefix") == 0)
@@ -412,10 +415,10 @@ main(int argc, char *argv[]) {
 				else if (strcasecmp(token, "chomp") == 0)
 					transforms |= TRANS_CHOMP;
 				else {
-					DESTROY(copy);
 					usage("unrecognized transform in -T");
 				}
-			DESTROY(copy);
+			}
+			tokstr_last(&ts);
 			break;
 		    }
 		case 'm':
@@ -1117,20 +1120,19 @@ batch_options(const char *optstr, qparam_t options, qparam_ct dflt) {
 	char **opt = optv;
 	const char *msg;
 	int optc, ch;
-	char *tok;
 
-	char *temp = strdup(optstr);
-	char *saveptr = NULL;
 	/* crack the option string based on space or tab delimiters. */
-	for (tok = strtok_r(temp, "\040\t", &saveptr);
-	     tok != NULL;
-	     tok = strtok_r(NULL, "\040\t", &saveptr))
+	struct tokstr *ts = tokstr_string(optstr);
+	for (char *tok;
+	     (tok = tokstr_next(ts, "\040\011")) != NULL;
+	     free(tok))
 	{
 		/* dispense with extra spaces and tabs (empty fields). */
 		if (*tok == '\0')
 			continue;
-		*opt++ = tok;
+		*opt++ = strdup(tok);
 	}
+	tokstr_last(&ts);
 
 	/* if no options were specified (e.g., $options\n), restore defaults.
 	 */
@@ -1174,8 +1176,10 @@ batch_options(const char *optstr, qparam_t options, qparam_ct dflt) {
 			qparam_debug("batch", options);
 	}
 	/* done. */
+
+	for (int i = 0; i < opt - optv; i++)
+		free(optv[i]);
 	DESTROY(optv);
-	DESTROY(temp);
 	return msg;
 }
 
@@ -1363,28 +1367,32 @@ query_launcher(qdesc_ct qdp, qparam_ct qpp, writer_t writer) {
 
 	/* ready player one. */
 	CREATE(query, sizeof(struct query));
+	query->descrip = makepath(qdp);
+	query->mode = qdp->mode;
+	query->params = *qpp;
+	qpp = NULL;
 
 	/* define the fence. */
-	if (qpp->after != 0) {
-		if (qpp->complete) {
+	if (query->params.after != 0) {
+		if (query->params.complete) {
 			/* each db tuple must begin after the fence-start. */
-			fence.first_after = qpp->after;
+			fence.first_after = query->params.after;
 		} else {
 			/* each db tuple must end after the fence-start. */
-			fence.last_after = qpp->after;
+			fence.last_after = query->params.after;
 		}
 	}
-	if (qpp->before != 0) {
-		if (qpp->complete) {
+	if (query->params.before != 0) {
+		if (query->params.complete) {
 			/* each db tuple must end before the fence-end. */
-			fence.last_before = qpp->before;
+			fence.last_before = query->params.before;
 		} else {
 			/* each db tuple must begin before the fence-end. */
-			fence.first_before = qpp->before;
+			fence.first_before = query->params.before;
 		}
 	}
 
-	/* branch on rrtype. */
+	/* branch on rrtype; launch (or queue) nec'y fetches. */
 	if (qdp->rrtype == NULL) {
 		/* no rrtype string given, let makepath set it to "any". */
 		char *path = makepath(qdp);
@@ -1397,12 +1405,11 @@ query_launcher(qdesc_ct qdp, qparam_ct qpp, writer_t writer) {
 		return NULL;
 	} else {
 		/* rrtype string was given, parse comma separated list. */
-		char *rrtypes = strdup(qdp->rrtype);
-		char *saveptr = NULL;
 		int nfetches = 0;
-		for (char *rrtype = strtok_r(rrtypes, ",", &saveptr);
-		     rrtype != NULL;
-		     rrtype = strtok_r(NULL, ",", &saveptr))
+		struct tokstr *ts = tokstr_string(qdp->rrtype);
+		for (char *rrtype;
+		     (rrtype = tokstr_next(ts, ",")) != NULL;
+		     free(rrtype))
 		{
 			struct qdesc qd = {
 				.mode = qdp->mode,
@@ -1416,19 +1423,16 @@ query_launcher(qdesc_ct qdp, qparam_ct qpp, writer_t writer) {
 			nfetches++;
 			DESTROY(path);
 		}
+		tokstr_last(&ts);
 		if (nfetches > 1)
 			query->multitype = true;
-		DESTROY(rrtypes);
 	}
 
 	/* finish query initialization, link it up, and return it. */
 	query->writer = writer;
 	writer = NULL;
-	query->params = *qpp;
 	query->next = query->writer->queries;
 	query->writer->queries = query;
-	query->descrip = makepath(qdp);
-	query->mode = qdp->mode;
 	return query;
 }
 
@@ -1437,19 +1441,18 @@ query_launcher(qdesc_ct qdp, qparam_ct qpp, writer_t writer) {
 static const char *
 rrtype_correctness(const char *input) {
 	char **rrtypeset = calloc(MAX_FETCHES, sizeof(char *));
-	char *rrtypes = strdup(input);
-	char *saveptr = NULL;
 	const char *ret = NULL;
 	int nrrtypeset = 0;
 	bool some = false, any = false,
 		some_dnssec = false, any_dnssec = false;
-	for (char *rrtype = strtok_r(rrtypes, ",", &saveptr);
-	     rrtype != NULL;
-	     rrtype = strtok_r(NULL, ",", &saveptr))
+	struct tokstr *ts = tokstr_string(input);
+	for (char *rrtype;
+	     (rrtype = tokstr_next(ts, ",")) != NULL;
+	     free(rrtype))
 	{
 		for (char *p = rrtype; *p != '\0'; p++)
-			if (isupper(*p))
-				*p = (char) tolower(*p);
+			if (isupper((int)*p))
+				*p = (char) tolower((int)*p);
 		if (nrrtypeset == MAX_FETCHES) {
 			ret = "too many rrtypes specified";
 			goto done;
@@ -1459,7 +1462,7 @@ rrtype_correctness(const char *input) {
 				ret = "duplicate rrtype encountered";
 				goto done;
 			}
-		rrtypeset[nrrtypeset++] = rrtype;
+		rrtypeset[nrrtypeset++] = strdup(rrtype);
 		if (strcmp(rrtype, "any") == 0)
 			any = true;
 		else if (strcmp(rrtype, "any-dnssec") == 0)
@@ -1474,9 +1477,11 @@ rrtype_correctness(const char *input) {
 			 strcmp(rrtype, "nsec3") == 0 ||
 			 strcmp(rrtype, "nsec3param") == 0 ||
 			 strcmp(rrtype, "dlv") == 0)
+		{
 			some_dnssec = true;
-		else
+		} else {
 			some = true;
+		}
 		if (any && some) {
 			ret = "ANY is redundant when mixed like this";
 			goto done;
@@ -1486,8 +1491,10 @@ rrtype_correctness(const char *input) {
 			goto done;
 		}
 	}
+	tokstr_last(&ts);
  done:
-	DESTROY(rrtypes);
+	for (int i = 0; i < nrrtypeset; i++)
+		DESTROY(rrtypeset[i]);
 	DESTROY(rrtypeset);
 	return ret;
 }
@@ -1496,10 +1503,7 @@ rrtype_correctness(const char *input) {
  */
 static void
 launch_fetch(query_t query, const char *path, pdns_fence_ct fp) {
-	qparam_ct qpp = &query->params;
-	char *url;
-
-	url = psys->url(path, NULL, qpp, fp, false);
+	char *url = psys->url(path, NULL, &query->params, fp, false);
 	if (url == NULL)
 		my_exit(1);
 
