@@ -81,7 +81,7 @@ unmake_curl(void) {
 	}
 }
 
-/* fetch -- given a url, tell libcurl to go fetch it.
+/* fetch -- given a url, tell libcurl to go fetch it, attach fetch to query.
  */
 fetch_t
 create_fetch(query_t query, char *url) {
@@ -159,6 +159,7 @@ fetch_reap(fetch_t fetch) {
 		curl_slist_free_all(fetch->hdrs);
 		fetch->hdrs = NULL;
 	}
+	DESTROY(fetch->saf_msg);
 	DESTROY(fetch->url);
 	DESTROY(fetch->buf);
 	DESTROY(fetch);
