@@ -29,6 +29,7 @@
 #include "ns_ttl.h"
 #include "pdns.h"
 #include "time.h"
+#include "tokstr.h"
 #include "globals.h"
 
 static void present_text_line(const char *, const char *, const char *);
@@ -128,14 +129,14 @@ present_text_lookup(pdns_tuple_ct tup,
  */
 static void
 present_text_line(const char *rrname, const char *rrtype, const char *rdata) {
-	char *asnum = NULL, *cidr = NULL, *comment = NULL;
-	const char *result = NULL;
+	char *asnum = NULL, *cidr = NULL, *comment = NULL, *result = NULL;
 
 #ifndef CRIPPLED_LIBC
 	result = asinfo_from_rr(rrtype, rdata, &asnum, &cidr);
 #endif
 	if (result != NULL) {
-		comment = strdup(result);
+		comment = result;
+		result = NULL;
 	} else if (asnum != NULL && cidr != NULL) {
 		const char *src = asnum;
 		bool wordbreak = true;
@@ -369,28 +370,26 @@ annotate_one(json_t *anno, const char *rdata, const char *name, json_t *obj) {
 #ifndef CRIPPLED_LIBC
 static json_t *
 annotate_asinfo(const char *rrtype, const char *rdata) {
-	char *asnum = NULL, *cidr = NULL;
+	char *asnum = NULL, *cidr = NULL, *result = NULL;
 	json_t *asinfo = NULL;
-	const char *result;
 
 	if ((result = asinfo_from_rr(rrtype, rdata, &asnum, &cidr)) != NULL) {
 		asinfo = json_object();
 		json_object_set_new_nocheck(asinfo, "comment",
 					    json_string(result));
+		free(result);
 	} else if (asnum != NULL && cidr != NULL) {
 		json_t *array = json_array();
-		char *copy, *walker, *token;
-
-		copy = walker = strdup(asnum);
-		while ((token = strsep(&walker, " ")) != NULL)
-			json_array_append(array, json_integer(atol(token)));
-		free(copy);
+		struct tokstr *ts = tokstr_string(asnum);
+		for (char *t; (t = tokstr_next(ts, "\040")) != NULL; free(t))
+			json_array_append_new(array, json_integer(atol(t)));
+		tokstr_last(&ts);
 		asinfo = json_object();
 		json_object_set_new_nocheck(asinfo, "as", array);
 		json_object_set_new_nocheck(asinfo, "cidr", json_string(cidr));
-		free(asnum);
-		free(cidr);
 	}
+	DESTROY(asnum);
+	DESTROY(cidr);
 	return asinfo;
 }
 #endif
@@ -467,15 +466,15 @@ present_csv_line(pdns_tuple_ct tup, const char *rdata) {
 		printf("\"%s\"", rdata);
 	if (asinfo_lookup && tup->obj.rrtype != NULL &&
 	    tup->obj.rdata != NULL) {
-		char *asnum = NULL, *cidr = NULL;
-		const char *result = NULL;
+		char *asnum = NULL, *cidr = NULL, *result = NULL;
 
 #ifndef CRIPPLED_LIBC
 		result = asinfo_from_rr(tup->rrtype, rdata, &asnum, &cidr);
 #endif
 		if (result != NULL) {
 			asnum = strdup(result);
-			cidr = strdup(result);
+			cidr = result;
+			result = NULL;
 		}
 		putchar(',');
 		if (asnum != NULL) {
